
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <malloc.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <signal.h>
#include <strings.h>
#include "protocolo.h"
#include "mysql.h"


int sock_S = 0;
MYSQL mysql;

/* ---------------------------------------------------------------------- */
/* CONTROLADOR: funcion que permite finalizar correctamente la ejecucion  */
/*              al pulsar crtl+c.                                         */
/* ---------------------------------------------------------------------- */

void controlador (int numero)
{
   if (sock_S != 0) {
      close(sock_S);
   }
 
   // Cerramos conexin BD.  
   mysql_close(&mysql);
   
	// Se pone controlador por defecto para ctrl-c
	signal (SIGINT, SIG_DFL);
   // Los procesos hijos que vuelvan a ser zombies
   signal (SIGCHLD, SIG_DFL);
   exit(1);
}

/* --------------------------------------------------------------------------------- */
/* MENSAJE_INICIAL_SERVIDOR: muestra los datos de conexion del servidor al iniciar   */
/* --------------------------------------------------------------------------------- */

void mensaje_inicial_servidor (char *nombre_servidor, struct sockaddr_in *info)
{
   printf("\n*** Bienvenido al Servidor de Ficheros HAMSTER *** \n\n");
   printf("Nombre del Host Servidor: %s \n",nombre_servidor);
   printf("Informacion del socket: \n");
   printf("\t - Numero de socket: %d \n",sock_S);
   printf("\t - Numero de puerto: %d \n",ntohs(info -> sin_port));
   printf("\t - Direccion del host: %s \n",(char *)inet_ntoa(info -> sin_addr));
   printf("\t - Familia: %d \n\n",info -> sin_family);
   printf("Servidor a la espera de peticiones de los clientes... \n\n");
}


/* -------------------------------------------------------------------------------------- */
/* ENVIAR_RESULTADOS_ENCOLADOS : envia al cliente los resultados obtenidos de diferentes  */
/*                               consultas realizadas por el servidor.                    */
/* -------------------------------------------------------------------------------------- */

void enviar_resultados_encolados(Cola *cola, struct sockaddr_in *dir_socket_cliente, int len_info_sock, 
                                 int search, int modo, int num_respuesta)
{
   struct TRegistroCompartido *archivo_compartido;
   char *nick;
   char *respuesta_OK = (char *)malloc(sizeof(char) * 4);
   char *respuesta_KO1 = (char *)malloc(sizeof(char) * 5);
   char *respuesta_KO2 = (char *)malloc(sizeof(char) * 5);
   
   sprintf(respuesta_OK,"OK%d",num_respuesta);
   sprintf(respuesta_KO1,"ERR%d",num_respuesta);
   sprintf(respuesta_KO2,"ERR%d",num_respuesta + 1);
   
   if (search == 1) {
      // error
      sendto(sock_S,respuesta_KO1,4,0,(struct sockaddr *)dir_socket_cliente,len_info_sock);
   } else if (search == 2) {
      // no encontrado ningun resultado
      sendto(sock_S,respuesta_KO2,4,0,(struct sockaddr *)dir_socket_cliente,len_info_sock);
   } else {
      // Resultado encontrados y guardados en cola
      sendto(sock_S,respuesta_OK,3,0,(struct sockaddr *)dir_socket_cliente,len_info_sock);

      // Cuantos resultados
      sendto(sock_S,&cola->total,sizeof(unsigned int),0,(struct sockaddr *)dir_socket_cliente,len_info_sock);
    
      // Enviamos cada uno de los resultados sin esperar respuesta del cliente
      int i;
      reiniciar_PDI(cola);
      for (i=0; i<cola->total; i++) {
         switch (modo) {
            case CONSULTA_FICHEROS_SERVIDOR:
            case CONSULTA_FICHEROS_USUARIO:
               archivo_compartido = (struct TRegistroCompartido *)obtener_elemento_PDI(cola);
#ifdef DEBUG
               fprintf(stderr, "Encontrado archivo %s del usuario: %s\n", archivo_compartido->archivo, 
                               archivo_compartido->nick);
               fflush(stderr);
#endif
               sendto(sock_S,archivo_compartido,sizeof(struct TRegistroCompartido),0,(struct sockaddr *)dir_socket_cliente,
                      len_info_sock);
               break;
            case MOSTRAR_USUARIOS_ACTIVOS:
               nick = (char *)obtener_elemento_PDI(cola);
               sendto(sock_S,nick,TAM_NICKS,0,(struct sockaddr *)dir_socket_cliente,len_info_sock);
               break;
         }
         avanzar_PDI(cola);
      }
      vaciar_cola(cola);
   }
}


/* ------------------- */
/* PROGRAMA PRINCIPAL  */
/* ------------------- */

int main (int argc, char **argv)
{
   int valor, temp, id;
   int len_info_sock;
   unsigned int puerto;
   char *nombre_servidor = NULL;
   struct sockaddr_in dir_socket_servidor;
   struct sockaddr_in dir_socket_cliente;
   char mens_cabecera[BUFSIZ];
   Cabecera *info_cabecera;
   char *IP_peticion;
   struct TInfoFichero *info_fichero = NULL;
   struct Tipo_CabeceraAlta *info_alta;
   char info_alta_puerto[6];
   struct TInfoUsuario *info_usuario;
   struct TRegistroCliente *info_cliente = NULL;
   
   // Variables de BD
   char *server="localhost", *user="root", *password="hola", *database="hamster";
   Cola *cola = NULL;
   //struct TRegistroCliente cliente;
   
   // Obtencion del parametro 'puerto'

   if (argc != 2) {
      fprintf(stderr,"Sintaxis: hamster_server puerto \n");
      fprintf(stderr,"\t (puerto: valor entre 0 y 65635) \n");
      exit(-1);
   }
   else {
      puerto = atoi((char *)argv[1]);
   }

   // Cambio de controlador para ctrl-c y asi finalizar correctamente el programa
   // al abortar el bucle de recepcion de paquetes con ctrl-c

   signal (SIGINT, controlador);
   
   // Que los procesos hijos no se queden zombies al terminar (la unica forma de
   // salir de zombie es haciendo un wait el padre, cosa que no nos interesa)
   signal(SIGCHLD,SIG_IGN);
   
   // Creacion del socket

   sock_S = socket(AF_INET,SOCK_DGRAM,0);
   if (sock_S == -1) {
      fprintf(stderr,"Error al crear socket en el servidor! \n");
      exit(-1);
   }
   
   // Obtencion del nombre y direccion del host local
   // y contruccion de la estructura sockaddr_in
   
   // NOTA: Escuchar en todas las Interfaces/IPs de la mquina (0.0.0.0)
   dir_socket_servidor.sin_port = htons(puerto);            // Puerto a utilizar
   dir_socket_servidor.sin_family = AF_INET;                // Protocolo Internet
   dir_socket_servidor.sin_addr.s_addr = INADDR_ANY;
   bzero(&(dir_socket_servidor.sin_zero),8);
   
   // Enlace del socket con el puerto a utilizar
   // Previamente se reusa la direccion del socket, de forma que no
   // sea necesario esperar un tiempo para usar de nuevo el pueto
   
   temp = 1;
   setsockopt(sock_S,SOL_SOCKET,SO_REUSEADDR,&temp,sizeof(temp));
   
   valor = bind(sock_S,(struct sockaddr *) &dir_socket_servidor,sizeof(dir_socket_servidor));
   if (valor == -1) {
      perror("Error al enlazar el socket con el puerto! \n");
      exit(-1);
   }
   mensaje_inicial_servidor(nombre_servidor,&dir_socket_servidor);

   // Conectamos con la BD e inicializamos las tablas
   mysql_conectar_db(&mysql, server, user, password, database);
   mysql_borrar_tablas(&mysql);
   mysql_crear_tablas(&mysql);
   
   // Ejecucion indefinida del servidor, hasta abortar su ejecucion con ctrl+c,
   // procesando concurrentemente todas las peticiones recibidas de los clientes
   
   len_info_sock = sizeof(dir_socket_cliente);
   while (1)
   {
      valor = recvfrom(sock_S,mens_cabecera,BUFSIZ,0,(struct sockaddr *)&dir_socket_cliente,
                       &len_info_sock);
      if (valor == -1) {
         perror("Error al recibir datos");
         exit(-1);
      }

      // Impresin en formato legible de la direccin IP del cliente contenida en
      // dir_socket_cliente.sin_addr.s_addr
#ifdef DEBUG
      fprintf(stderr, "Conexin cliente: %s\n", inet_ntoa(dir_socket_cliente.sin_addr));
      fflush(stderr);
#endif
      /////
      
      id = fork();         // Creacion de un proceso hijo para atender la peticion cliente
      switch (id) {
         case 0:                // Proceso hijo
            info_cabecera = (Cabecera *)mens_cabecera;
            IP_peticion = inet_ntoa(dir_socket_cliente.sin_addr);
            switch (info_cabecera -> accion) {
               case ALTA_EN_SERVIDOR:
                  //alta_usuario(nueva_IP);
                  
                  info_alta = (struct Tipo_CabeceraAlta *)mens_cabecera;
                  // Convertimos int a string
                  snprintf(info_alta_puerto, 6, "%i", info_alta->puerto);
                  if (mysql_alta_cliente(&mysql, IP_peticion, info_alta_puerto, info_alta->nick)==1) {
                     sendto(sock_S,"ERR1",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  } else {
                     sendto(sock_S,"OK1",3,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }
#ifdef DEBUG
                  fprintf(stderr, "* Alta %s\n", IP_peticion);
                  fflush(stderr);
#endif
                  break;
                  
               case COMPARTIR_FICHERO:
                  info_fichero = (struct TInfoFichero *)mens_cabecera;
                  
                  if (mysql_cliente_dado_de_alta(&mysql, info_fichero->nick)) {
                     if (mysql_insertar_archivo_compartido(&mysql, info_fichero->nick, info_fichero->nombre)==1) {
                        sendto(sock_S,"ERR2",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                     } else {
                        sendto(sock_S,"OK2",3,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                     }
                  } else {
                     sendto(sock_S,"ERR2",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }
                  
#ifdef DEBUG
                     fprintf(stderr, "- %s\n", info_fichero->nombre);
                     fflush(stderr);
#endif
                  break;
                  
               case BAJA_EN_SERVIDOR:
                  if (mysql_cliente_dado_de_alta(&mysql, info_cabecera->nick)) {
                     if (mysql_baja_cliente(&mysql, info_cabecera->nick)==1) {
                        sendto(sock_S,"ERR3",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                     } else {
                        sendto(sock_S,"OK3",3,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                     }
                  } else {
                     sendto(sock_S,"ERR3",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }
                  
#ifdef DEBUG
                  fprintf(stderr, "* Baja %s\n", IP_peticion);
                  fflush(stderr);
#endif
                  break;
                  
               case CONSULTA_FICHEROS_SERVIDOR:
                  // Consultar los ficheros del servidor similares al nombre proporcionado
                  if (mysql_cliente_dado_de_alta(&mysql, info_cabecera->nick)) {
                     info_fichero = (struct TInfoFichero *)mens_cabecera;
                     cola = (Cola *)crear_cola();
                     int search = mysql_buscar_archivo(&mysql, info_fichero->nombre, cola);
                     int num_respuesta = 4;
                     int modo = CONSULTA_FICHEROS_SERVIDOR;
                     enviar_resultados_encolados(cola,&dir_socket_cliente,len_info_sock,search,modo,num_respuesta);
                  } else {
                     sendto(sock_S,"ERR4",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }
                  
                  break;

               case MOSTRAR_USUARIOS_ACTIVOS:
                  // Obtener lista de usuarios y enviarla al cliente
                  if (mysql_cliente_dado_de_alta(&mysql, info_cabecera->nick)) {
                     cola = (Cola *)crear_cola();
                     int search = mysql_listar_usuarios(&mysql, cola);
                     int num_respuesta = 6;
                     int modo = MOSTRAR_USUARIOS_ACTIVOS;
                     enviar_resultados_encolados(cola,&dir_socket_cliente,len_info_sock,search,modo,num_respuesta);
                  } else {
                     sendto(sock_S,"ERR6",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }
                  break;

               case CONSULTA_FICHEROS_USUARIO:
                  // Obtener lista de ficheros de la IP cliente especificada y enviarla al cliente
                  if (mysql_cliente_dado_de_alta(&mysql, info_cabecera->nick)) {
                     info_usuario = (struct TInfoUsuario *)mens_cabecera;
                     cola = (Cola *)crear_cola();
                     int search = mysql_archivos_del_cliente(&mysql, info_usuario->nick_interes, cola);
                     int num_respuesta = 8;
                     int modo = CONSULTA_FICHEROS_USUARIO;
                     enviar_resultados_encolados(cola,&dir_socket_cliente,len_info_sock,search,modo,num_respuesta);
                  } else {
                     sendto(sock_S,"ERR8",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }
                  break;

               case CONSULTA_DATOS_USUARIO:
                  // Devolver los datos (ip,puerto) del nick cliente especificado
                  if (mysql_cliente_dado_de_alta(&mysql, info_cabecera->nick)) {
                     info_usuario = (struct TInfoUsuario *)mens_cabecera;
                     info_cliente = (struct TRegistroCliente *)malloc(sizeof (struct TRegistroCliente));
                     int search = mysql_info_cliente(&mysql, info_usuario->nick_interes, info_cliente);
                     if (search == 0) {
                        sendto(sock_S,"OK10",4,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                        sendto(sock_S,info_cliente,sizeof(struct TRegistroCliente),0,(struct sockaddr *)&dir_socket_cliente,
                               len_info_sock);
                     }
                     else {
                        sendto(sock_S,"ERR10",5,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                     }
                  } else {
                     sendto(sock_S,"ERR10",5,0,(struct sockaddr *)&dir_socket_cliente,len_info_sock);
                  }                     
                  break;
            }
            close(sock_S); // Peticin atendida
            exit(0);   // Finaliza hijo
            
            break;
            
         default:              // Proceso padre
            break;
      }
   }
   
   return (0);   
}
