#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <malloc.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "protocolo.h"
#include "files.h"
#include "cola.h"

#define BACKLOG 10     // how many pending connections queue will hold

int sockfd; // Socket del demonio de transferencia de arhivos (global para poder
            // cerrarlo al ser interrumpido)

/**
   Si el mensaje indicado es de error retorna el nmero del error (que siempre es 
   diferente de 0).
   Si no es mensaje de error retorna 0
**/
int es_error(char *msg) {
   if ((msg[0] == 'E') && (msg[1] == 'R') && (msg[2] == 'R')) {
      // No permite hacer una conversion con atoi ya que solo es 1 caracter,
      // asi que lo hago manualmente
      switch(msg[3]) {
         case '1':
            return 1;
         case '2':
            return 2;
         case '3':
            return 3;
         case '4':
            return 4;
         case '5':
            return 5;
         case '6':
            return 6;
         case '7':
            return 7;
         case '8':
            return 8;
         case '9':
            return 9;
         default:
            return 0;   // Error
      }
   } else {
      return 0;
   }
}

/**
   Si el mensaje indicado es de error retorna el nmero del error (que siempre es 
   diferente de 0).
   Si no es mensaje de error retorna 0
**/
int es_ok(char *msg) {
   if ((msg[0] == 'O') && (msg[1] == 'K')) {
      // No permite hacer una conversion con atoi ya que solo es 1 caracter,
      // asi que lo hago manualmente
      switch(msg[2]) {
         case '1':
            return 1;
         case '2':
            return 2;
         case '3':
            return 3;
         case '4':
            return 4;
         case '5':
            return 5;
         case '6':
            return 6;
         case '7':
            return 7;
         case '8':
            return 8;
         case '9':
            return 9;
         default:
            return 0;   // Error
      }
   } else {
      return 0;
   }
}

/**
   Busca un nombre de archivo en una cola de ficheros
   Devuelve 1 si encuentra el archivo especificado en la cola
**/
int archivo_compartido(Cola *ficheros, char *archivo) {
   reiniciar_PDI(ficheros);
   int i;
   for (i=0; i<ficheros->total; i++) {
      if (strcmp(archivo, (char *)obtener_elemento_PDI(ficheros)) == 0) {
         return 1;
      }
      avanzar_PDI(ficheros);
   }
   return 0;
}


/**
   Funcin ejecutado por el proceso que ejecuta file_transfer_daemon una evz
   recibe un SIGINT
**/
void controlador_file_transfer (int numero)
{
   close(sockfd); // Cerramos socket del filetransfer
   
	// Se pone controlador por defecto para ctrl-c
	signal (SIGINT, SIG_DFL);
   
   // Los procesos hijos que vuelvan a ser zombies
   signal (SIGCHLD, SIG_DFL);
   
   exit(1);
}


/**
   Crea socket global para que sea utilizado por fil_transfer_daemon, elige como
   puerto origen uno aleatorio y lo devuelve
   Devuelve 1 en caso de error, de lo contrario devuelve el puerto seleccionado.
**/
unsigned int file_transfer_socket() {
   // int sockfd -> es global
   struct sockaddr_in my_addr;    // my address information
   unsigned int puerto;
   
   if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
     perror("socket");
     return 1;
   }
   
   my_addr.sin_family = AF_INET;         // host byte order
   my_addr.sin_port = 0;     // puerto aleatorio
   my_addr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
   memset(&(my_addr.sin_zero), '\0', 8); // zero the rest of the struct
   
   if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) == -1) {
     perror("bind");
     return 1;
   }
   
   // Obtenemos el puerto que bind nos ha asignado
   socklen_t len = sizeof(struct sockaddr);
   getsockname(sockfd, (struct sockaddr *)&my_addr, &len);
   //puerto = (unsigned int *)malloc(sizeof(unsigned int));
   puerto = (unsigned int)ntohs(my_addr.sin_port);
#ifdef DEBUG
   fprintf(stderr, "Puerto: %i\n", ntohs(my_addr.sin_port));
   fflush(stderr);
#endif
   
   if (listen(sockfd, BACKLOG) == -1) {
     perror("listen");
     return 1;
   }
   
   return puerto;
}
    
/**
   Atiende peticiones de archivos por el puerto TCP FILE_TRANSFER_PORT.
   Comprueba que el archivo solicitado exista y este siendo compartido.
   En caso de error envia al cliente un "Not Found", en caso de xito se
   envia el archivo.
   ** Es encesario ejecutar file_transfer_socket antes que esta para crear
   ** el socket global.
**/
void file_transfer_daemon(Cola *ficheros) {
   // int sockfd -> es global
   int new_fd;  // listen on sock_fd, new connection on new_fd
   int sin_size;
   char nombre_archivo[TAM_NOMBRE_ARCHIVO];
   int archivo, bytes;
   char buffer[BUFSIZ];
   struct stat estado;
   int filesize;
   struct sockaddr_in their_addr; // connector's address information
   
   // Si recibimos SIGINT paramos
   signal (SIGINT, controlador_file_transfer);
   
   // Que los procesos hijos no se queden zombies al terminar (la unica forma de
   // salir de zombie es haciendo un wait el padre, cosa que no nos interesa)
   signal(SIGCHLD,SIG_IGN);
   
   while(1) {
     sin_size = sizeof(struct sockaddr_in);
     if ((new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size)) == -1) {
         perror("accept");
         continue;
     }
   #ifdef DEBUG
     fprintf(stderr, "Transfer File Daemon: Nueva conexin de %s\n",inet_ntoa(their_addr.sin_addr));
     fflush(stderr);
   #endif
     if (!fork()) { // Hijo
         close(sockfd); // El hijo no necesita escuchar
         
         // Recibimos nombre de archivo que solicita
         read(new_fd, &nombre_archivo, TAM_NOMBRE_ARCHIVO);
         
         // El fichero solicitado esta siendo compartido?
         if (! archivo_compartido(ficheros, nombre_archivo)) {
            // No esta compartido! Error
            if (send(new_fd, "Not Found\n", 10, 0) == -1) {
               perror("send");
            }
            close(new_fd);
            exit(0);
         }
         
         // Miramos el tamao del fichero
         if (stat(nombre_archivo, &estado)) {
            // No existe! Error
            if (send(new_fd, "Not Found\n", 10, 0) == -1) {
               perror("send");
            }
            close(new_fd);
            exit(0);
         } else {
            // Enviamos tamao del fichero
            filesize = estado.st_size;
            snprintf(buffer, BUFSIZ, "%i", filesize);
            if (send(new_fd, buffer, sizeof buffer, 0) == -1) {
               perror("send");
            }
         }
         
         
         archivo = open(nombre_archivo, O_RDONLY);
         if (archivo == -1) {
            // No existe! Error
            if (send(new_fd, "Not Found\n", 10, 0) == -1) {
               perror("send");
            }
         } else {
            while ((bytes = read(archivo, buffer, BUFSIZ))){
               if (send(new_fd, buffer, bytes, 0) == -1) {
                  perror("send");
               }
            }
         }
         
         close(new_fd);
         close(archivo);
         exit(0);
     }
     close(new_fd);  // El padre no necesita el nuevo socket del hijo
   }

}

/**
   Solicita a una IP/Host un archivo. Si se descarga con xito devuelve un 0,
   de lo contrario un 1.
   Las descargas se realizan en el directorio "incoming/". Si ya existe un archivo
   con el mismo nombre lo reescribe.
**/
int pedir_archivo(char *host, int puerto, char *nombre_archivo) {
    int sockfd, bytes;  
    char buffer[BUFSIZ];
    struct hostent *he;
    struct sockaddr_in their_addr; 
    char *ptr;
    char archivo_local[TAM_NOMBRE_ARCHIVO] = "incoming";
    int archivo, filesize, total_filesize=0;
    int bytes_a_leer;

    if ((he=gethostbyname(host)) == NULL) { 
        perror("gethostbyname");
        exit(1);
    }

    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        exit(1);
    }

    their_addr.sin_family = AF_INET;    // host byte order 
    their_addr.sin_port = htons(puerto);  // short, network byte order 
    their_addr.sin_addr = *((struct in_addr *)he->h_addr);
    memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct 

    if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) {
        perror("connect");
        return 1;
    }

    // Enviamos archivo que solicitamos
    if (send(sockfd, nombre_archivo, TAM_NOMBRE_ARCHIVO, 0) == -1) {
       perror("send");
    }
    
    // Recibimos Tamao
    if ((bytes=recv(sockfd, buffer, sizeof buffer, 0)) == -1) {
        perror("recv");
        return 1;
    }
    // El archivo solicitado no ha sido encontrado
    if (strcmp(buffer, "Not Found\n")==0) {
        return 1;
    }
    filesize = atoi(buffer);

    // Coger solo el nombre del fichero que descargamos, descartamos el directorio
    // donde esta guardado en la mquina remota
    ptr = strrchr(nombre_archivo, '/');
    // Construimos "incoming/nombre_del_archivo" para guardar en ese directorio
    strncat(archivo_local, ptr, TAM_NOMBRE_ARCHIVO- sizeof("incoming"));
    
    // Creamos archivo, si existe se sobreescribe.
    if ((archivo = open(archivo_local, O_WRONLY|O_TRUNC|O_CREAT, 0666)) == -1) {
       perror("Error al crear archivo");
       return 1;
    }
    
    
    //////
    // Recibimos los primeros bytes, debemos comprobar que no se nos devuelva
    // un not found por error al abrirlo en el servidor.
    if (filesize - total_filesize <= sizeof(buffer)) {
       bytes_a_leer = filesize - total_filesize;
    } else {
       bytes_a_leer = sizeof(buffer);
    }
    if ((bytes=recv(sockfd, buffer, bytes_a_leer, 0)) == -1) {
        perror("recv");
        close(archivo);
        return 1;
    }
    // El archivo solicitado no ha sido encontrado
    if (strcmp(buffer, "Not Found\n")==0) {
        return 1;
    }
    total_filesize += bytes;
    write(archivo, buffer, bytes);
    //////
    
    // Recibimos el resto del archivo
    while (total_filesize < filesize) {
       if (filesize - total_filesize <= sizeof(buffer)) {
         bytes_a_leer = filesize - total_filesize;
       } else {
         bytes_a_leer = sizeof(buffer);
       }
       if ((bytes=recv(sockfd, buffer, bytes_a_leer, 0)) == -1) {
        perror("recv");
        close(archivo);
        return 1;
       }
       total_filesize += bytes;
       write(archivo, buffer, bytes);
    }
    //////

    close(sockfd);
    close(archivo);
    return 0;
}

/**
   Recorre toda la cola de ficheros para ir aadiendo uno a uno al servidor.
   Devuelve 1 en caso de error
**/
int CompartirFicheros(int sock, struct sockaddr_in dir_socket_servidor, char *nick, Cola *ficheros){
   struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   struct TInfoFichero info_fichero;
   char resp[4];

   info_fichero.accion = COMPARTIR_FICHERO;
   snprintf(info_fichero.nick, TAM_NICKS, "%s", nick);   

   // Envio de los nombres de ficheros
   reiniciar_PDI(ficheros);
   int i;
   for (i=0; i<ficheros->total; i++) {
      snprintf(info_fichero.nombre, TAM_NOMBRE_ARCHIVO, "%s", obtener_elemento_PDI(ficheros));
      /*Enviamos*/
      sendto(sock, &info_fichero, sizeof(struct TInfoFichero),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
      /*Recibimos*/
      recvfrom(sock,resp,4,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
#ifdef DEBUG
      fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]);
#endif

      if (es_ok(resp) != 2) {
         // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
         // peticin. En principio nunca deberia suceder esto.
         return 1;
      } else if (es_error(resp) == 2) {
         return 1;
      }
      
      avanzar_PDI(ficheros);
   }
   return 0;
}


/**
   Nos da de alta en el servidor
   Devuelve 1 en caso de error
**/
int Alta(int sock, struct sockaddr_in dir_socket_servidor, char *nick, unsigned int puerto){
   struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   struct Tipo_CabeceraAlta info;
   char resp[4];

   snprintf(info.nick, TAM_NICKS, "%s", nick);
   info.accion = ALTA_EN_SERVIDOR;   
   info.puerto = puerto;

   /*Enviamos*/
   sendto(sock, &info, sizeof(struct Tipo_CabeceraAlta),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
   /*Recibimos*/
   recvfrom(sock,resp,4,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
   
#ifdef DEBUG
   fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]);
#endif

   if (es_ok(resp) == 1) {
      return 0;
   } else if (es_error(resp) == 1) {
      return 1;
   } else {
      // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
      // peticin. En principio nunca deberia suceder esto.
      return 1;
   }

}


/**
   Nos da de baja en el servidor
   Devuelve 1 en caso de error
**/
int Baja(int sock, struct sockaddr_in dir_socket_servidor, char *nick){
   struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   Cabecera info;
   char resp[4];

   snprintf(info.nick, TAM_NICKS, "%s", nick);
   info.accion = BAJA_EN_SERVIDOR;   

   /*Enviamos*/
   sendto(sock, &info, sizeof(Cabecera),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
   /*Recibimos*/
   recvfrom(sock,resp,4,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));

#ifdef DEBUG
   fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]);
#endif

   if (es_ok(resp) == 3) {
      return 0;
   } else if (es_error(resp) == 3) {
      return 1;
   } else {
      // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
      // peticin. En principio nunca deberia suceder esto.
      return 1;
   }
}

/**
   Busca un archivo en el servidor, en la cola se aaden todos los resultados
   Devuelve 2 en caso de no encontrar nada
   Devuelve 1 en caso de error
**/
int Buscar_archivo(int sock, struct sockaddr_in dir_socket_servidor, char *nick, char *archivo, Cola *cola){
   struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   struct TInfoFichero info;
   char resp[4];
   char buffer[BUFSIZ];
   unsigned int encontrados;
   struct TRegistroCompartido *archivo_encontrado_map;
   struct TRegistroCompartido *archivo_encontrado;
   
   snprintf(info.nick, TAM_NICKS, "%s", nick);
   info.accion = CONSULTA_FICHEROS_SERVIDOR;
   snprintf(info.nombre, TAM_NOMBRE_ARCHIVO, "%s", archivo);

   /*Enviamos*/
   sendto(sock, &info, sizeof(struct TInfoFichero),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
   /*Recibimos*/
   recvfrom(sock,resp,4,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
   
#ifdef DEBUG
   fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]); fflush(stdout);
#endif

   if (es_error(resp) == 4) {
      return 1;   // Error
   } else if (es_error(resp) == 5) {
      return 2;   // Ningun resultado
   } else if (es_ok(resp) == 4) {
      
      recvfrom(sock,&encontrados, sizeof(unsigned int),0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));

#ifdef DEBUG
      fprintf(stderr, "Encontrados: %i\n", encontrados);
#endif      
      int i;
      for (i=0; i<encontrados; i++){
         recvfrom(sock,buffer, sizeof(struct TRegistroCompartido),0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
         
         archivo_encontrado_map = (struct TRegistroCompartido *) buffer;
         
         // Copiamos la info obtenida del buffer a una nueva zona de memoria ya que
         // el buffer sera reescrito en la siguiente iteracin.
         archivo_encontrado = (struct TRegistroCompartido *) malloc(sizeof(struct TRegistroCompartido));
         snprintf(archivo_encontrado->nick, TAM_NICKS, "%s", archivo_encontrado_map->nick);
         snprintf(archivo_encontrado->archivo, TAM_NOMBRE_ARCHIVO, "%s", archivo_encontrado_map->archivo);

#ifdef DEBUG
         fprintf(stderr, "Encontrado... Usuario: %s Archivo: %s \n", archivo_encontrado->nick, archivo_encontrado->archivo);
#endif      
         // Segun la interfaz que se use con el usuario, aqui podriamos imprimir
         // los resultados en algun sitio en lugar de meterlo en una cola para tratarlo
         // despues
         encolar(cola, (char *)archivo_encontrado);
      }
      return 0;   // Hay resultados
   } else {
      // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
      // peticin. En principio nunca deberia suceder esto.
      return 1;
   }

}

/**
   Lista los usuarios dados de alta en el servidor, en la cola se aaden todos los resultados
   Devuelve 2 en caso de no encontrar nada
   Devuelve 1 en caso de error
**/
int Listar_usuarios(int sock, struct sockaddr_in dir_socket_servidor, char *nick, Cola *cola){
   struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   struct Tipo_Cabecera info;
   char resp[4];
   char buffer[BUFSIZ];
   unsigned int encontrados;
   char *usuario;
   
   snprintf(info.nick, TAM_NICKS, "%s", nick);
   info.accion = MOSTRAR_USUARIOS_ACTIVOS;

   /*Enviamos*/
   sendto(sock, &info, sizeof(struct Tipo_Cabecera),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
   /*Recibimos*/
   recvfrom(sock,resp,4,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
   
#ifdef DEBUG
   fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]);
#endif

   if (es_error(resp) == 6) {
      return 1;   // Error
   } else if (es_error(resp) == 7) {
      return 2;   // Ningun resultado
   } else if (es_ok(resp) == 6) {
      
      recvfrom(sock,&encontrados, sizeof(unsigned int),0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));

#ifdef DEBUG
      fprintf(stderr, "Encontrados: %i\n", encontrados);
#endif      
      int i;
      for (i=0; i<encontrados; i++){
         recvfrom(sock,buffer, TAM_NICKS,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
         
         // Copiamos la info obtenida del buffer a una nueva zona de memoria ya que
         // el buffer sera reescrito en la siguiente iteracin.
         usuario = (char *) malloc(TAM_NICKS);
         snprintf(usuario, TAM_NICKS, "%s", buffer);

#ifdef DEBUG
         fprintf(stderr, "Usuario: %s\n", usuario);
#endif      
         // Segun la interfaz que se use con el usuario, aqui podriamos imprimir
         // los resultados en algun sitio en lugar de meterlo en una cola para tratarlo
         // despues
         encolar(cola, (char *)usuario);
      }
      return 0;   // Hay resultados
   } else {
      // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
      // peticin. En principio nunca deberia suceder esto.
      return 1;
   }

}


/**
   Crea un socket y un sockaddr_in para poder realizar las conexiones al servidor indicado
**/
void crear_socket(char *nombre_servidor, unsigned int puerto, int *sock, struct sockaddr_in *dir_socket_servidor) {
   struct sockaddr_in dir_socket_cliente;    /*estructura:puerto,@IP,protocolo*/
   
   struct hostent *info_host_S;      /*estructura con info del host Serv*/ 
   struct in_addr ** addr_host_S;    /*direccion IP del host servidor*/ 
   
   /*sock = (int *)malloc(sizeof(int));
   dir_socket_servidor = (struct sockaddr_in *)malloc(sizeof(struct sockaddr_in));*/
   
   info_host_S= (struct hostent *) malloc (sizeof(struct hostent)); 
   addr_host_S= (struct in_addr **) malloc (sizeof(struct in_addr)*10);
   
// Creacion del socket

   *sock = socket(AF_INET,SOCK_DGRAM,0);
   if (*sock == -1) {
      fprintf(stderr,"Error al crear socket en el servidor! \n");
      exit(-1);
   }
   
   /*obtenemos direccion local del host*/
   
   // NOTA: Que permita recibir datos por cualquier interfaz/ip ya que el
   dir_socket_cliente.sin_addr.s_addr = INADDR_ANY;
   dir_socket_cliente.sin_port= 0;                /*puerto 0: se reseva uno libre*/  
   dir_socket_cliente.sin_family= AF_INET;        /*protocolo que usamos*/     
   
   /*Dejamos reusar la direccion de socket.Sin esta instruccion hemos*/ 
   /*de esperar un tiempo para poder usar de nuevo el puerto*/ 
   int tmp=1; 
   setsockopt(*sock,SOL_SOCKET,SO_REUSEADDR,&tmp,sizeof(tmp)); 
   
 
   /*enlazamos socket con puerto*/ 
   if ((bind(*sock,(struct sockaddr *) &dir_socket_cliente, sizeof(dir_socket_cliente)))==-1) { 
      fprintf(stderr, "Error Cliente: Conectar a puerto"); 
      perror((char *)bind); 
      exit(1);
    } 
   
   /*obtenemos direccion del host servidor*/ 
   if ((info_host_S=gethostbyname((char *)nombre_servidor))==NULL) { 
      fprintf(stderr, "Error cliente:Obtener direccion servidor"); 
      perror((char *) gethostbyname); 
      exit(1); 
    } 
  
   addr_host_S= (struct in_addr **)info_host_S->h_addr_list; 
   
   dir_socket_servidor->sin_addr= ** addr_host_S;  /*Direccion del server*/    
   dir_socket_servidor->sin_port= htons(puerto);  /*Puerto del server*/   
   dir_socket_servidor->sin_family= AF_INET;
}
   
   /**
   Lista los archivos del usuario con nick especificado (cliente), en la cola se aaden todos los resultados
   Devuelve 2 en caso de no encontrar nada
   Devuelve 1 en caso de error
**/
int Listar_archivos_cliente(int sock, struct sockaddr_in dir_socket_servidor, char *nick, char *cliente, Cola *cola) {
   struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   struct TInfoUsuario info;
   char resp[4];
   char buffer[BUFSIZ];
   unsigned int encontrados;
   char *archivo;
   
   snprintf(info.nick_interes, TAM_NICKS, "%s", cliente);
   snprintf(info.nick, TAM_NICKS, "%s", nick);
   info.accion = CONSULTA_FICHEROS_USUARIO;
      
   /*Enviamos*/
   sendto(sock, &info, sizeof(struct TInfoUsuario),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
   /*Recibimos*/
   recvfrom(sock,resp,5,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));
   
#ifdef DEBUG
   fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]);
#endif

   if (es_error(resp) == 8) {
      return 1;   // Error
   } else if (es_error(resp) == 9) {
      return 2;   // Ningun resultado
   } else if (es_ok(resp) == 8) {
      
      recvfrom(sock,&encontrados, sizeof(unsigned int),0,(struct sockaddr *)&dir_socket_recibido,
               (socklen_t *)sizeof(dir_socket_recibido));
#ifdef DEBUG
      fprintf(stderr, "Encontrados: %i\n", encontrados);
#endif      
      int i;
      for (i=0; i<encontrados; i++) {
         recvfrom(sock,buffer,TAM_NOMBRE_ARCHIVO,0,(struct sockaddr *)&dir_socket_recibido,
                  (socklen_t *)sizeof(dir_socket_recibido));
         
         // Copiamos la info obtenida del buffer a una nueva zona de memoria ya que
         // el buffer sera reescrito en la siguiente iteracin.
         archivo = (char *) malloc(TAM_NOMBRE_ARCHIVO);
         snprintf(archivo, TAM_NOMBRE_ARCHIVO, "%s", buffer);

#ifdef DEBUG
         fprintf(stderr, "Archivo: %s\n", archivo);
#endif      
         encolar(cola, (char *) archivo);
      }
      return 0;   // Hay resultados
   } 
   else {
      // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
      // peticin. En principio nunca deberia suceder esto.
      return 1;
   }

}

/**
   Obtiene los datos (IP y puerto) del cliente con nick especificado
   La estructura TInfoCliente debe existir previamente
   Devuelve 1 en caso de error
**/
int Consultar_datos_cliente(int sock, struct sockaddr_in dir_socket_servidor, char *nick, char *cliente, 
                            struct TRegistroCliente *info_cliente) {
      struct sockaddr_in dir_socket_recibido;  /*info de quien recibimos*/
   struct TInfoUsuario info;
   char resp[4];
   
   snprintf(info.nick_interes, TAM_NICKS, "%s", cliente);
   snprintf(info.nick, TAM_NICKS, "%s", nick);
   info.accion = CONSULTA_DATOS_USUARIO;
      
   /*Enviamos*/
   sendto(sock, &info, sizeof(struct TInfoUsuario),0,(struct sockaddr *)&dir_socket_servidor,sizeof(dir_socket_servidor));
   /*Recibimos*/
   recvfrom(sock,resp,5,0,(struct sockaddr *)&dir_socket_recibido,(socklen_t *)sizeof(dir_socket_recibido));

#ifdef DEBUG
   fprintf(stderr, "%c%c%c\n", resp[0], resp[1], resp[2]);
#endif

   if (es_error(resp) == 1) {
      return 1;   // Error
   } else if (es_ok(resp) == 1) {
      
      recvfrom(sock,info_cliente, sizeof(struct TRegistroCliente),0,(struct sockaddr *)&dir_socket_recibido,
               (socklen_t *)sizeof(dir_socket_recibido));
      
      return 0;   // Datos recibidos correctamente
   } 
   else {
      // Nos han devuelto un mensaje con un cdigo que no corresponde con nuestra 
      // peticin. En principio nunca deberia suceder esto.
      return 1;
   }

}


