#include <stdio.h>
#include <stdlib.h>
#include <mysql/mysql.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <dirent.h>
#include <string.h>
#include "cola.h"
#include "protocolo.h"
#include "mysql.h"



/**
   Conecta a la BD segun los parametros indicados. Establece las variables
   globales para poder trabajar sobre la conexin.
   Devuelve 1 en caso de error.
   */
int mysql_conectar_db (MYSQL *mysql, char *server, char *user, char *password, char *database) {
   
   mysql_init(mysql);
   if ((mysql_real_connect(mysql, server, user, password, database, 0, NULL, 0))==NULL){
#ifdef DEBUG
      fprintf(stderr, "Imposible conectarse al motor!\n\n%s\n\n", mysql_error(mysql));
      perror("");
#endif
      return 1;
   }
   return 0;
}

/**
   Ejecuta la sentencia SQL que le pasamos por parmetro
**/
int mysql_ejecutar_sql (MYSQL *mysql, char *sql) {

#ifdef DEBUG
      fprintf(stderr, "SQL: %s\n", sql);
#endif

   //if (mysql_real_query(mysql, sql, strlen(sql))!=0 ) {
   if (mysql_query(mysql, sql)!=0 ) {
#ifdef DEBUG
      fprintf(stderr, "Error ejecutando sql (%s)\n", mysql_error(mysql));
#endif
      return 1;
   }
   return 0;
}

/**
   Crea tabla en la BD para guardar IPs y Archivos.
   Devuelve 1 en caso de error
   */
int mysql_crear_tablas (MYSQL *mysql) {
   char *ibuff;
   size_t ibuff_size = 100 * sizeof(char);
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   snprintf(ibuff, ibuff_size, "CREATE TABLE compartido (nick VARCHAR(%i), archivo VARCHAR(%i))", TAM_NICKS, TAM_NOMBRE_ARCHIVO);
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   snprintf(ibuff, ibuff_size, "CREATE TABLE sesiones (ip VARCHAR(16), puerto INT, nick VARCHAR(%i))", TAM_NICKS);
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   free(ibuff);
   return 0;
}

/**
   Borra una tabla de la BD
   Devuelve 1 en caso de error
   **/
int mysql_ejecutar_drop_table (MYSQL *mysql, char *tabla) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(tabla) * sizeof(char);
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }
   
   snprintf(ibuff, ibuff_size, "DROP TABLE %s", tabla);
#ifdef DEBUG
   fprintf(stderr, "%s\n", ibuff);
#endif

   int ret = mysql_ejecutar_sql(mysql, ibuff);
   free(ibuff);
   
   return ret;
}

/**
   Borra las tablas que utiliza hamster de la BD
   Devuelve 1 en caso de error
   **/
int mysql_borrar_tablas (MYSQL *mysql) {
   if (mysql_ejecutar_drop_table(mysql, "compartido") == 1) {
      return 1;
   }
   if (mysql_ejecutar_drop_table(mysql, "sesiones") == 1) {
      return 1;
   }
   return 0;
}


/**
   Inserta un registro con una nick y un archivo determinado en la tabla indicada
   Devuelve 1 en caso de error
   **/
int mysql_insertar_archivo_compartido (MYSQL *mysql, char *nick, char *archivo) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(nick) * sizeof(char) + strlen(archivo) * sizeof(char);
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   snprintf(ibuff, ibuff_size, "INSERT INTO compartido (nick, archivo) VALUES ('%s', '%s')", nick, archivo);
   
   int ret = mysql_ejecutar_sql(mysql, ibuff);
   
   free(ibuff);
   return ret;
}


/**
   Inserta un registro con una IP, puerto y nick en la tabla de sesiones
   Devuelve 1 en caso de error
   **/
int mysql_alta_cliente (MYSQL *mysql, char *ip, char *puerto, char *nick) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(ip) * sizeof(char) + strlen(puerto) * sizeof(char) + strlen(nick) * sizeof(char);
   
   // Si el cliente ya estaba dado de alta, suponemos que el cliente ha perdido
   // la conexin y no pudo darse de baja, asi que borramos sus datos antiguos
   if (mysql_cliente_dado_de_alta(mysql, nick)) {
      mysql_baja_cliente(mysql, nick);
   } 
   
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   snprintf(ibuff, ibuff_size, "INSERT INTO sesiones (ip, puerto, nick) VALUES ('%s', '%s','%s')", ip, puerto, nick);

   int ret = mysql_ejecutar_sql(mysql, ibuff);
   free(ibuff);
   return ret;
}

/**
   Borra un registro con el nick indicado en la tabla de sesiones
   Devuelve 1 en caso de error
   **/
int mysql_baja_cliente (MYSQL *mysql, char *nick) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(nick) * sizeof(char);

   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   snprintf(ibuff, ibuff_size, "DELETE FROM sesiones WHERE nick = '%s'", nick);
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   snprintf(ibuff, ibuff_size, "DELETE FROM compartido WHERE nick = '%s'", nick);
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   free(ibuff);
   return 0;
}


/**
   Lista los usuarios dados de alta (solo nicks)
   Devuelve 1 en caso de error
   Si no es encontrado devuelve 2.
   */
int mysql_listar_usuarios (MYSQL *mysql, Cola *cola) {
   char *ibuff;
   size_t ibuff_size = 100;
   MYSQL_RES *res;
   MYSQL_ROW row;
   unsigned int num_fields;
   char *cliente;
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   // 37 = numero ascii del smbolo %
   snprintf(ibuff, ibuff_size, "SELECT nick FROM sesiones");

   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   // Obtenermos resultado
   if (!(res=mysql_store_result(mysql))){
#ifdef DEBUG
      fprintf(stderr,"Couldn't get result: %s\n", mysql_error(mysql));
#endif
      free(ibuff);
      return 1;
   }
   
   if (res)  // Hay resultado
   {
      num_fields = mysql_num_fields(res);
      if ( !(row = mysql_fetch_row(res))) {
         return 2;
      }
      // Por cada resultado
      do{
         cliente = (char *)malloc(TAM_NICKS);
         snprintf(cliente, TAM_NICKS, "%s", row[0]);
#ifdef DEBUG
         fprintf(stderr, "Nick: %s", cliente);
#endif
         encolar(cola, cliente);
      } while ((row = mysql_fetch_row(res)));
   } else {
      free(ibuff);
      return 2;
   }
    
   mysql_free_result(res);
   free(ibuff);
   return 0;
}

/**
   Busca en una tabla el registro que contenga el archivo indicado, todos los
   registros encontrados son introducidos en la cola especificada.
   Devuelve 1 en caso de error
   Si no es encontrado devuelve 2.
   */
int mysql_buscar_archivo (MYSQL *mysql, char *archivo, Cola *cola) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(archivo) * sizeof(char);
   MYSQL_RES *res;
   MYSQL_ROW row;
   unsigned int num_fields;
   struct TRegistroCompartido *registro;
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   // 37 = numero ascii del smbolo %
   snprintf(ibuff, ibuff_size, "SELECT * FROM compartido WHERE archivo LIKE '%c%s%c'", 37,archivo,37);

   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   // Obtenermos resultado
   if (!(res=mysql_store_result(mysql))){
#ifdef DEBUG
      fprintf(stderr,"Couldn't get result: %s\n", mysql_error(mysql));
#endif
      free(ibuff);
      return 1;
   }

   if (res)  // Hay resultado
   {
      num_fields = mysql_num_fields(res);
      if (!(row = mysql_fetch_row(res))) {
         return 2;
      }
      // Por cada resultado
      do {
         registro = (struct TRegistroCompartido *)malloc(sizeof(struct TRegistroCompartido));
         snprintf(registro->nick, TAM_NICKS, "%s", row[0]);
         snprintf(registro->archivo, TAM_NOMBRE_ARCHIVO, "%s", row[1]);
#ifdef DEBUG
         fprintf(stderr, "Nick: %s, Archivo: %s\n", registro->nick, registro->archivo);
#endif
         encolar(cola, (char *)registro);

      } while ((row = mysql_fetch_row(res)));
   } else {
      free(ibuff);
      return 2;
   }
   
   mysql_free_result(res);
   free(ibuff);
   return 0;
}

/**
   Busca los archivos que pertenecen a un nick cliente, todos los
   registros encontrados son introducidos en la cola especificada.
   Devuelve 1 en caso de error
   Si no es encontrado devuelve 2.
   */
int mysql_archivos_del_cliente (MYSQL *mysql, char *nick, Cola *cola) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(nick) * sizeof(char);
   MYSQL_RES *res;
   MYSQL_ROW row;
   unsigned int num_fields;
   char *archivo;
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   snprintf(ibuff, ibuff_size, "SELECT archivo FROM compartido WHERE nick LIKE '%s'", nick);
   
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   // Obtenermos resultado
   if (!(res=mysql_store_result(mysql))){
#ifdef DEBUG
      fprintf(stderr,"Couldn't get result: %s\n", mysql_error(mysql));
#endif
      free(ibuff);
      return 1;
   }
   
   if (res)  // Hay resultado
   {
      num_fields = mysql_num_fields(res);
      // Por cada resultado
      while ((row = mysql_fetch_row(res))) {
         archivo = (char *)malloc(sizeof(char) * TAM_NOMBRE_ARCHIVO);
         snprintf(archivo, TAM_NOMBRE_ARCHIVO, "%s", row[0]);
#ifdef DEBUG
         fprintf(stderr, "Archivo: %s\n", archivo);
#endif
         encolar(cola, archivo);
      }
   } else {
      free(ibuff);
      return 2;
   }
    
   mysql_free_result(res);
   free(ibuff);
   return 0;
}


/**
   Busca informacin sobre un nick: su ip y puerto. Se devolver en una estructura
   cliente (esta debe existir ya antes de pasarla como parmetro).
   En caso de error devuelve 1.
   Si no es encontrado devuelve 2.
**/
int mysql_info_cliente (MYSQL *mysql, char *nick, struct TRegistroCliente *cliente) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(nick) * sizeof(char);
   MYSQL_RES *res;
   MYSQL_ROW row;
   unsigned int num_fields;
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 1;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   snprintf(ibuff, ibuff_size, "SELECT * FROM sesiones WHERE nick LIKE '%s'", nick);
   
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 1;
   }
   
   // Obtenermos resultado
   if (!(res=mysql_store_result(mysql))){
#ifdef DEBUG
      fprintf(stderr,"Couldn't get result: %s\n", mysql_error(mysql));
#endif
      free(ibuff);
      return 1;
   }
   
   if (res)  // Hay resultado
   {
      num_fields = mysql_num_fields(res);
      // Solo debe haber 1 resultado posible ya que los nicks son nicos
      if ((row = mysql_fetch_row(res))) {
         snprintf(cliente->ip, 16, "%s", row[0]);
         snprintf(cliente->puerto, 6, "%s", row[1]);
#ifdef DEBUG
         fprintf(stderr, "IP: %s Puerto: %s\n", cliente->ip, cliente->puerto);
#endif
      }
   } else {
      free(ibuff);
      return 2;
   }
    
   mysql_free_result(res);
   free(ibuff);
   return 0;
}

/**
   Consulta si un nick ha sido dada de alta.
   Si ha sido dada de alta devuelve 1, sino 0.
   En caso de error devuelve 2.
**/
int mysql_cliente_dado_de_alta (MYSQL *mysql, char *nick) {
   char *ibuff;
   size_t ibuff_size = 100 + strlen(nick) * sizeof(char);
   MYSQL_RES *res;
   MYSQL_ROW row;
   
   /* Guardamos espacio para la longitud de los valores + las instrucciones */
   ibuff=(char *)malloc(ibuff_size);

   if (ibuff==NULL) {
#ifdef DEBUG
      perror("Asignacion de memoria para inserccion insuficiente");
#endif
      return 2;
   }

   // No se controla ataques por inyeccin SQL, mirar funcin mysql_real_escape_string(...)
   snprintf(ibuff, ibuff_size, "SELECT * FROM sesiones WHERE nick LIKE '%s'", nick);
   
   if (mysql_ejecutar_sql(mysql, ibuff) == 1) {
      free(ibuff);
      return 2;
   }
   
   // Obtenermos resultado
   if (!(res=mysql_store_result(mysql))){
#ifdef DEBUG
      fprintf(stderr,"Couldn't get result: %s\n", mysql_error(mysql));
#endif
      free(ibuff);
      return 2;
   }
   
   if (res) {
      // Si hay resultado
      if ((row = mysql_fetch_row(res))) {
         mysql_free_result(res);
         free(ibuff);
         return 1;
      } else {
         mysql_free_result(res);
         free(ibuff);
         return 0;
      }
   } else {
      free(ibuff);
      return 0;
   }

}


// Prueba del funcionamiento de las funciones anteriores
/*int main (int argc, char *argv[]){
   // Es necesario que la BD hamster ya este creada en el servidor
   char *server="localhost", *user="root", *password="hola", *database="hamster";
   Cola *cola = (Cola *)crear_cola();
   struct TRegistroCliente cliente;
   MYSQL mysql;

   mysql_conectar_db(&mysql, server, user, password, database);

   mysql_borrar_tablas(&mysql);
   mysql_crear_tablas(&mysql);
   mysql_alta_cliente(&mysql, "127.0.0.1", "5555", "localhost");
   mysql_insertar_archivo_compartido(&mysql, "localhost", "/tmp/hola.txt");
   mysql_insertar_archivo_compartido(&mysql, "localhost", "/tmp/adios.txt");
   mysql_insertar_archivo_compartido(&mysql, "localhost", "/tmp/hamster.txt");
   mysql_insertar_archivo_compartido(&mysql, "localhost", "/tmp/sharp.txt");
   mysql_buscar_archivo(&mysql, "txt", cola);
   mysql_archivos_del_cliente(&mysql, "localhost", cola);
   mysql_info_cliente(&mysql, "localhost", &cliente);
   if (mysql_cliente_dado_de_alta(&mysql, "localhost")) {
      printf("localhost existe!\n");
   } else {
      printf("localhost NO existe!\n");
   }
   if (mysql_cliente_dado_de_alta(&mysql, "saturno")) {
      printf("saturno existe!\n");
   } else {
      printf("saturno NO existe!\n");
   }
   mysql_baja_cliente(&mysql, "localhost");
   mysql_close(&mysql);
   return 0;
}*/
